#include "inc/exports.h"

#ifdef VSH_MENU
#include "vsh_menu.h"
#endif

#ifdef GAME_MENU
#include "game_menu.h"
#endif

SYS_MODULE_INFO (PS3_MENU, 0, 1, 0);
SYS_MODULE_START(ps3_menu_start);
SYS_MODULE_STOP (ps3_menu_stop);

#define THREAD_NAME         "ps3_menu_thread"
#define STOP_THREAD_NAME    "ps3_menu_stop_thread"

static sys_ppu_thread_t ps3_menu_tid = -1;
static bool done = 0;
extern int32_t menu_running;
int32_t ps3_menu_start(uint64_t arg);
int32_t ps3_menu_stop(void);

/***********************************************************************
* sys_ppu_thread_exit, direct over syscall
***********************************************************************/
static inline void _sys_ppu_thread_exit(uint64_t val)
{
    system_call_1(41, val);
}

/***********************************************************************
*
***********************************************************************/
static inline sys_prx_id_t prx_get_module_id_by_address(void *addr)
{
    system_call_1(461, (uint64_t)(uint32_t)addr);
    return (int32_t)p1;
}

/***********************************************************************
* plugin main ppu thread
***********************************************************************/
static void ps3_menu_thread(uint64_t arg) {
  #ifdef VSH_MENU
  vsh_menu();
  #elif GAME_MENU
  game_menu();
  #endif
  sys_ppu_thread_exit(0);
}

/***********************************************************************
* start thread
***********************************************************************/
int32_t ps3_menu_start(uint64_t arg)
{
    sys_ppu_thread_create(&ps3_menu_tid, ps3_menu_thread, 0, 3000, 0x4000, 1, THREAD_NAME);
    _sys_ppu_thread_exit(0);
    return SYS_PRX_RESIDENT;
}

/***********************************************************************
* stop thread
***********************************************************************/
static void ps3_menu_stop_thread(uint64_t arg)
{
    done = 1;

    if(ps3_menu_tid != (sys_ppu_thread_t)-1)
    {
        uint64_t exit_code;
        sys_ppu_thread_join(ps3_menu_tid, &exit_code);
    }

    sys_ppu_thread_exit(0);
}


/***********************************************************************
*
***********************************************************************/
static void finalize_module(void)
{
    uint64_t meminfo[5];

    sys_prx_id_t prx = prx_get_module_id_by_address(finalize_module);

    meminfo[0] = 0x28;
    meminfo[1] = 2;
    meminfo[3] = 0;

    system_call_3(482, prx, 0, (uint64_t)(uint32_t)meminfo);
}


/***********************************************************************
*
***********************************************************************/
int ps3_menu_stop(void)
{
    sys_ppu_thread_t t;
    uint64_t exit_code;

    sys_ppu_thread_create(&t, ps3_menu_stop_thread, 0, 0, 0x2000, 1, STOP_THREAD_NAME);
    sys_ppu_thread_join(t, &exit_code);

    finalize_module();
    _sys_ppu_thread_exit(0);
    return SYS_PRX_STOP_OK;
}
